//Call dibs on slots for sounds
freeslot("sfx_fstep0", "sfx_kstep1")

//Decide on the player's feet sounds based on their current skin
addHook("PlayerThink", function(player)
	if not player.mo
		return
	end
	
	// Assign feet sounds to each of the default characters
	// Order of sounds is: Landing thud (optional: fast landing thud), walking footsteps, running footsteps
	local footsounds = {}
	footsounds["sonic"] = {{nil}, {nil}, {nil}}
	footsounds["tails"] = {{nil, nil}, {nil}, {sfx_putput}}
	footsounds["knuckles"] = {{sfx_s3k4c}, {sfx_kstep1}, {sfx_kstep1}}
	
	// Pick feet sounds for the player based on their skin, default to Sonic's
	player.footsounds = footsounds[player.mo.skin] or footsounds["sonic"]
end)

addHook("ThinkFrame", function()
	// Landing thud effect function
	local function landingThud(player)
		
		local thudsound = nil
		// Maximum landing thud volume is reached when you hit the ground at thudmax * FRACUNIT
		local thudmax = 5
		// Scale landing thud sound to how fast they hit the ground
		// prevmomz should be negative, except in antigrav
		local thudvolume = 255 * max(0, min(abs(player.prevmomz), thudmax*FRACUNIT) / thudmax / FRACUNIT)
		
		// Player is moving on foot slowly or not at all
		if player.mo.state == S_PLAY_STND or player.mo.state == S_PLAY_WALK
			// Play the slow landing thud
			thudsound = player.footsounds[1][1]
		// Player is, or was, moving on foot quickly
		elseif player.mo.state == S_PLAY_RUN or player.mo.state == S_PLAY_SKID
			// Default to slow landing thud if there isn't a fast thud
			thudsound = player.footsounds[1][2] or player.footsounds[1][1]
		// Player is doing something else so their feet shouldn't be hitting the ground
		else
			return
		end
		
		S_StartSoundAtVolume(player.mo, thudsound or sfx_kc50, thudvolume or 255)
	end
	
	// Footstep effect function
	local function footSteps(player)
		// Only play a sound at the start of an animation frame
		if player.mo.frame == player.prevframe
			return
		end
		
		local stepsound = nil
		local stepvolume = 255
		local minvolume = 102
		
		if player.mo.state == S_PLAY_WALK
			// Play footstep sound on frames when the foot is hitting the ground
			if (player.mo.frame == 2 or player.mo.frame == 6)
				// Randomly pick one of available walking footstep sounds
				stepsound = player.footsounds[2][P_RandomRange(1, table.getn(player.footsounds[2]))]
			else
				return
			end
			// Scale footstep volume to how close player is to running animation
			stepvolume = 255 * player.speed / player.runspeed
		elseif player.mo.state == S_PLAY_RUN
			// Play footstep sound on frames when the foot is hitting the ground
			if (player.mo.skin != "tails" and (player.mo.frame == 1 or player.mo.frame == 3))
			// Special condition for Tails to make the putput sound effect sound right
			or (player.mo.skin == "tails" and (leveltime % 10 == 0 or leveltime % 10 == 1))
				//Randomly pick one of available running footstep sounds
				stepsound = player.footsounds[3][P_RandomRange(1, table.getn(player.footsounds[3]))]
			else
				return
			end
		// Not walking or running, no footsteps
		else
			return
		end
		
		// If anything went wrong, default to default footstep at 255
		S_StartSoundAtVolume(player.mo, stepsound or sfx_fstep0, max(stepvolume, minvolume) or 255)
	end
	for player in players.iterate()
		// If spectator, get outta here
		if not player.mo
			continue
		end
		
		// Gotta be on the ground for your feet to make noises
		if P_IsObjectOnGround(player.mo) == true
			if player.prevground == false
				// Landing thud effects
				landingThud(player)
			else
				// Footstep effects
				footSteps(player)
			end
		end
		//Extra clause for Tails so he still putputs through the air
		if player.mo.skin == "tails" and player.mo.state == S_PLAY_RUN
			footSteps(player)
		end
		
		//Set up vars for next frame's checks
		player.prevframe = player.mo.frame
		player.prevground = P_IsObjectOnGround(player.mo)
		player.prevmomz = player.mo.momz
	end
end, MT_PLAYER)
